from __future__ import annotations
from typing import Iterable, Optional

import numpy as np
from tcod.console import Console

import tile_types
from entity import Entity

class FloorMap:
	def __init__(self, width: int, height: int, entities: Iterable[Entity] = ()):
		#terrain stuff
		self.width, self.height = width, height
		self.tiles = np.full((width, height), fill_value=tile_types.wall, order="F")

		self.visible = np.full((width, height), fill_value=False, order="F")
		self.explored = np.full((width, height), fill_value=False, order="F")

		#contents stuff
		self.entities = set(entities)
		self.procgen_cache = {} #reserved for the procgen algorithm, otherwise ignored

		#set externally
		self.engine = None
		self.player = None

	def in_bounds(self, x: int, y: int) -> bool:
		return 0 <= x < self.width and 0 <= y < self.height

	def get_entity_at(self, x: int, y: int, unwalkable_only: bool = False) -> Optional[Entity]:
		for entity in self.entities:
			if entity.x == x and entity.y == y:
				if unwalkable_only:
					if not entity.walkable:
						return entity
				else:
					return entity

		return None

	def render(self, console: Console) -> None:
		console.rgb[0:self.width, 0:self.height] = np.select(
			condlist = [self.visible, self.explored],
			choicelist = [self.tiles["light"], self.tiles["dark"]],
			default = tile_types.SHROUD
		)

		for entity in self.entities:
			if self.visible[entity.x, entity.y]:
				console.print(entity.x, entity.y, entity.char, fg=entity.color)